(*
 * Doubly-linked lists.
 *
 * ----------------------------------------------------------------
 *
 * @begin[license]
 * Copyright (C) 2003 Mojave Group, Caltech
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Jason Hickey
 * @email{jyh@cs.caltech.edu}
 * @end[license]
 *)

(*
 * Type of elements in the list.
 *)
type 'a elem

(*
 * The head of the list.
 *)
type 'a t

(*
 * Create a new empty list.
 *)
val create_list : unit -> 'a t

(*
 * Create a new unattached element.
 *)
val create_elem : 'a -> 'a elem

(*
 * Get the item for an element.
 *)
val item : 'a elem -> 'a

(*
 * Links.
 *)
val pred : 'a elem -> 'a elem option
val succ : 'a elem -> 'a elem option

(*
 * Unlink the element from the list.
 *)
val unlink : 'a elem -> unit

(*
 * Add the element to a list.
 *)
val insert : 'a t -> 'a elem -> unit

(*
 * Iterate over elements.
 *)
val iter : 'a t -> ('a elem -> unit) -> unit

(*
 * Fold over elements.
 *)
val fold : 'a t -> ('b -> 'a elem -> 'b) -> 'b -> 'b

(*
 * Find an element in the list.
 *)
val find : 'a t -> ('a elem -> bool) -> 'a elem

(*
 * Check if the list is empty.
 *)
val is_empty : 'a t -> bool

(*
 * Get the first element in the list.
 *)
val head : 'a t -> 'a elem

(*!
 * @docoff
 *
 * -*-
 * Local Variables:
 * Caml-master: "compile"
 * End:
 * -*-
 *)
